//*****************************************************************************
// File Name	: i2c7seg.c
// 
// Title		: I2C -> 7SEG LED slave
// Revision		: 1.0
// Notes		: 
// Target MCU	: Atmel AVR series ATtiny2313
// Tool Chain	: WinAVR-20081205
// 
// 
// Charlieplexing 7seg driver code refered to 
// 		http://www.instructables.com/id/Charlieplexing-7-segment-displays/
// 
// Revision History:
// When			Who			Description of change
// -----------	-----------	-----------------------
// 14-Mar-2009	S.Takagi	Created the program
//*****************************************************************************

//----- Include Files ---------------------------------------------------------
#include <inttypes.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>

#include "usiTwiSlave.h"
static const uint8_t TWI_slaveAddress = 0x33;

#define ESC (0x1B)
#define CHARLIE_1_D	PD0
#define CHARLIE_2_D	PD1
#define CHARLIE_3_D	PD2
#define CHARLIE_4_D	PD3
#define CHARLIE_5_D	PD4
#define CHARLIE_6_D	PD5
#define CHARLIE_7_D	PD6
#define CHARLIE_8_B	PB0
#define CHARLIE_9_B	PB1

#define LED_dp	7
#define LED_g	6
#define LED_f	5
#define LED_e	4
#define LED_d	3
#define LED_c	2
#define LED_b	1
#define LED_a	0

volatile uint8_t digit[16];
static uint8_t NumberOfDigits = 9 - 1; // 0-origin

unsigned char __attribute__ ((progmem)) LedChar[] = {
	0x3F, 0x06, 0x5b, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 	// 01234567
	0x7F, 0x6F, 0x77, 0x7C, 0x39, 0x5E, 0x79, 0x71, 	// 89AbCdEF
	0x00, 0x40, 0x48, 0x49, 0x08, 0x01, 0x41, 0x09, 	//  -=_~=
	0x5C, 0x63, 0x1C, 0x62, 0x58, 0x36, 0x7B, 0x76  	// cIIeH
};


ISR(TIMER0_OVF_vect)
{
	static uint8_t current_digit = 0;
	uint8_t segment;

	segment = digit[current_digit];

	// set ports direction to 'input'
	DDRD &= ~(
				(1<<CHARLIE_1_D) |
				(1<<CHARLIE_2_D) |
				(1<<CHARLIE_3_D) |
				(1<<CHARLIE_4_D) |
				(1<<CHARLIE_5_D) |
				(1<<CHARLIE_6_D) |
				(1<<CHARLIE_7_D)
			);

	DDRB &= ~(
				(1<<CHARLIE_8_B) |
				(1<<CHARLIE_9_B)
			);

	// set ports value to 'LOW'
	PORTD &= ~(
				(1<<CHARLIE_1_D) |
				(1<<CHARLIE_2_D) |
				(1<<CHARLIE_3_D) |
				(1<<CHARLIE_4_D) |
				(1<<CHARLIE_5_D) |
				(1<<CHARLIE_6_D) |
				(1<<CHARLIE_7_D)
			);

	PORTB &= ~(
				(1<<CHARLIE_8_B) |
				(1<<CHARLIE_9_B)
			);

	// set corresponding port to 'output' to sink
	if (segment & 0b00000001) DDRD |= (1<<CHARLIE_1_D);
	if (segment & 0b00000010) DDRD |= (1<<CHARLIE_2_D);
	if (segment & 0b00000100) DDRD |= (1<<CHARLIE_3_D);
	if (segment & 0b00001000) DDRD |= (1<<CHARLIE_4_D);
	if (segment & 0b00010000) DDRD |= (1<<CHARLIE_5_D);
	if (segment & 0b00100000) DDRD |= (1<<CHARLIE_6_D);
	if (segment & 0b01000000) DDRD |= (1<<CHARLIE_7_D);
	if (segment & 0b10000000) DDRB |= (1<<CHARLIE_8_B);

	// bit-fiddling
	switch (current_digit) {
	case 0:
		if (segment & 0b00000001) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_1_D);
		PORTD |= (1<<CHARLIE_1_D);
		break;
	case 1:
		if (segment & 0b00000010) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_2_D);
		PORTD |= (1<<CHARLIE_2_D);
		break;
	case 2:
		if (segment & 0b00000100) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_3_D);
		PORTD |= (1<<CHARLIE_3_D);
		break;
	case 3:
		if (segment & 0b00001000) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_4_D);
		PORTD |= (1<<CHARLIE_4_D);
		break;
	case 4:
		if (segment & 0b00010000) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_5_D);
		PORTD |= (1<<CHARLIE_5_D);
		break;
	case 5:
		if (segment & 0b00100000) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_6_D);
		PORTD |= (1<<CHARLIE_6_D);
		break;
	case 6:
		if (segment & 0b01000000) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_7_D);
		PORTD |= (1<<CHARLIE_7_D);
		break;
	case 7:
		if (segment & 0b10000000) DDRB |= (1<<CHARLIE_9_B);
		DDRB  |= (1<<CHARLIE_8_B);
		PORTB |= (1<<CHARLIE_8_B);
		break;
	case 8:
		DDRB  |= (1<<CHARLIE_9_B);
		PORTB |= (1<<CHARLIE_9_B);
		break;
	}


	if(++current_digit > NumberOfDigits)
		current_digit = 0;

}

#define HOME_POSITION 0

enum { S_NORMAL = 0, S_WRITE };

int main()
{
	unsigned char i, j;
	uint8_t work;
	static uint8_t pos = HOME_POSITION;
	static uint8_t pos_limit;
	static uint8_t rbuf[4], *p, temp;
	static uint8_t state = S_NORMAL;
//	static uint8_t semaphore = 0xFF; // busy

	// set I/O
	DDRB |= (1<<PB2);	// port pull-up
	DDRB |= (1<<PB3);
	DDRB |= (1<<PB4);
	PORTB |= (1<<PB2);	// port pull-up
	PORTB |= (1<<PB3);
	PORTB |= (1<<PB4);
	DDRB &= ~(1<<PB2);	// port input
	DDRB &= ~(1<<PB3);
	DDRB &= ~(1<<PB4);


	// I2C Slave initialization
	usiTwiSlaveInit( TWI_slaveAddress );

	// set timer
	TCCR0B = (1<<CS00)|(1<<CS01);
	TCCR0B = (1<<CS00);
	TIMSK |= (1<<TOIE0);
	TCNT0 = 0xFF;

	work = (PINB & ((1<<PB4)|(1<<PB3)|(1<<PB2)));
	work = (work>>PB2);
	NumberOfDigits = work + 1;
	pos_limit = NumberOfDigits;

	sei();

	// opening animation
	for (i = 0; i < 8; i++) {
		for (j = 0; j < 9; j++) {
			digit[j] = 1<<i;
			_delay_ms(10);
			digit[j] = 0;
		}
	}

	// main loop
	while(1) {

		sleep_mode();

		if( usiTwiDataInReceiveBuffer() ) {
	        temp = usiTwiReceiveByte();

			switch (state) {
			case S_NORMAL:
				switch (temp>>6) {
				case 1: // put encoded LED charaster from home position
					pos = HOME_POSITION;
				case 0: // put encoded LED charaster w/ post position increment
					digit[pos] = pgm_read_byte(LedChar + (temp & 0b00011111));
					if (temp & 0b00100000)
						digit[pos] |= (1<<LED_dp);
					if(++pos > pos_limit)
						pos = HOME_POSITION;
					break;
				case 2: // write operations
					p = rbuf;
					*p = temp;
				case 3:	// read operations
					work = temp & 0x0F;					// position preprocessing
					if (work > NumberOfDigits)
						work = NumberOfDigits;
					switch (temp & 0xF0) {
					case 0b11000000:					// read current position
						usiTwiTransmitByte(pos);
						break;
					case 0b11010000:					// read current position data
						usiTwiTransmitByte(digit[pos]);
						break;
					case 0b11100000:					// read certain position data
						usiTwiTransmitByte(digit[work]);
						break;
					case 0b11110000:					// read pos_limit
						usiTwiTransmitByte(pos_limit);
						break;
					case 0b10000000:
						state = S_WRITE;				// for Custom write wait next byte
						break;
					case 0b10010000:					// set position
						pos = work;
						break;
					case 0b10110000:					// set position limit
						pos_limit = work;
						break;
					}
					break;
				}
				break;
			case S_WRITE:
				*++p = temp;
				work = rbuf[0] & 0x0F;		// position preprocessing
				if (work > NumberOfDigits)
					work = NumberOfDigits;
				digit[work] = rbuf[1];
				state = S_NORMAL;
				break;
			}
	    }

	}

	return 0;
}
